@@ -5,7 +5,7 @@
 # SPDX-License-Identifier: MIT
 
 # NOTE (mb 2019-05-16): This module is substantially shared with the
-#   markdown-katex package and meaningful changes should be
+#   markdown-svgbob package and meaningful changes should be
 #   replicated there also.
 
 import os
@@ -36,21 +36,22 @@ PKG_BIN_DIR      = LIBDIR / "bin"
 FALLBACK_BIN_DIR = pl.Path("/") / "usr" / "local" / "bin"
 FALLBACK_BIN_DIR = FALLBACK_BIN_DIR.expanduser()
 
+CMD_NAME = "katex"
+
+# https://pymotw.com/3/platform/
+OSNAME  = platform.system()
+MACHINE = platform.machine()
+
+
 # NOTE (mb 2020-06-19): I have no idea if this is true and have not found a good
 #   way to test it, especially not in any cross platform way. Maybe KaTeX doesn't
 #   care and just uses the same encoding for input as for output.
 KATEX_INPUT_ENCODING  = "UTF-8"
 KATEX_OUTPUT_ENCODING = "UTF-8"
 
-CMD_NAME = "katex"
-
 # local cache so we don't have to validate the command every time
 TMP_LOCAL_CMD_CACHE = TMP_DIR / "local_katex_cmd.txt"
 
-# https://pymotw.com/3/platform/
-OSNAME  = platform.system()
-MACHINE = platform.machine()
-
 
 def _get_env_paths() -> typ.Iterable[pl.Path]:
     env_path = os.environ.get('PATH')
@@ -161,6 +162,10 @@ ArgValue = typ.Union[str, int, float, bool]
 Options  = typ.Dict[str, ArgValue]
 
 
+class KatexError(Exception):
+    pass
+
+
 def _iter_cmd_parts(options: Options = None) -> typ.Iterable[str]:
     for cmd_part in get_bin_cmd():
         yield cmd_part
@@ -189,11 +194,8 @@ def _cmd_digest(tex: str, cmd_parts: typ.List[str]) -> str:
     return hasher.hexdigest()
 
 
-class KatexError(Exception):
-    pass
-
-
 def _write_tex2html(cmd_parts: typ.List[str], tex: str, tmp_output_file: pl.Path) -> None:
+    # pylint: disable=consider-using-with ; not supported on py27
     tmp_input_file = TMP_DIR / tmp_output_file.name.replace(".html", ".tex")
     input_data     = tex.encode(KATEX_INPUT_ENCODING)
 
@@ -202,23 +204,33 @@ def _write_tex2html(cmd_parts: typ.List[str], tex: str, tmp_output_file: pl.Path
         fobj.write(input_data)
 
     cmd_parts.extend(["--input", str(tmp_input_file), "--output", str(tmp_output_file)])
-    proc     = sp.Popen(cmd_parts, stdout=sp.PIPE, stderr=sp.PIPE)
-    ret_code = proc.wait()
-    if ret_code < 0:
-        signame = SIG_NAME_BY_NUM[abs(ret_code)]
-        err_msg = (
-            f"Error processing '{tex}': "
-            + "katex_cli process ended with "
-            + f"code {ret_code} ({signame})"
-        )
-        raise KatexError(err_msg)
-    elif ret_code > 0:
-        stdout  = read_output(proc.stdout)
-        errout  = read_output(proc.stderr)
-        output  = (stdout + "\n" + errout).strip()
-        err_msg = f"Error processing '{tex}': {output}"
-        raise KatexError(err_msg)
-
+    proc = None
+    try:
+        proc     = sp.Popen(cmd_parts, stdout=sp.PIPE, stderr=sp.PIPE)
+        ret_code = proc.wait()
+        if ret_code < 0:
+            signame = SIG_NAME_BY_NUM[abs(ret_code)]
+            err_msg = (
+                f"Error processing '{tex}': "
+                + "katex_cli process ended with "
+                + f"code {ret_code} ({signame})"
+            )
+            raise KatexError(err_msg)
+        elif ret_code > 0:
+            stdout  = read_output(proc.stdout)
+            errout  = read_output(proc.stderr)
+            output  = (stdout + "\n" + errout).strip()
+            err_msg = f"Error processing '{tex}': {output}"
+            raise KatexError(err_msg)
+    finally:
+        if proc is not None:
+            # It might be reasonable that Popen itself raises an
+            # exception. In such a case, proc would still be None
+            # and there is nothing to close.
+            if proc.stdout is not None:
+                proc.stdout.close()
+            if proc.stderr is not None:
+                proc.stderr.close()
     tmp_input_file.unlink()
 
 
@@ -279,10 +291,16 @@ DEFAULT_HELP_TEXT = DEFAULT_HELP_TEXT.replace("\n", " ").replace("NL", "\n")
 
 
 def _get_cmd_help_text() -> str:
+    # pylint: disable=consider-using-with ; not supported on py27
     bin_parts = get_bin_cmd()
     cmd_parts = bin_parts + ['--help']
-    proc      = sp.Popen(cmd_parts, stdout=sp.PIPE)
-    help_text = read_output(proc.stdout)
+    proc      = None
+    try:
+        proc      = sp.Popen(cmd_parts, stdout=sp.PIPE)
+        help_text = read_output(proc.stdout)
+    finally:
+        if proc is not None and proc.stdout is not None:
+            proc.stdout.close()
     return help_text
 
 
